"""
Sphinx plugins for Django documentation.
"""

import re

from docutils import nodes
from docutils.parsers.rst import Directive
from docutils.statemachine import ViewList
from sphinx import addnodes
from sphinx import version_info as sphinx_version
from sphinx.directives.code import CodeBlock
from sphinx.domains.std import Cmdoption
from sphinx.util import logging
from sphinx.writers.html import HTMLTranslator

logger = logging.getLogger(__name__)
# RE for option descriptions without a '--' prefix
simple_option_desc_re = re.compile(r"([-_a-zA-Z0-9]+)(\s*.*?)(?=,\s+(?:/|-|--)|$)")


def setup(app):
    app.add_crossref_type(
        directivename="setting",
        rolename="setting",
        indextemplate="pair: %s; setting",
    )
    app.add_crossref_type(
        directivename="templatetag",
        rolename="ttag",
        indextemplate="pair: %s; template tag",
    )
    app.add_crossref_type(
        directivename="templatefilter",
        rolename="tfilter",
        indextemplate="pair: %s; template filter",
    )
    app.add_crossref_type(
        directivename="fieldlookup",
        rolename="lookup",
        indextemplate="pair: %s; field lookup type",
    )
    app.add_object_type(
        directivename="django-admin",
        rolename="djadmin",
        indextemplate="pair: %s; django-admin command",
        parse_node=parse_django_admin_node,
    )
    app.add_directive("django-admin-option", Cmdoption)
    app.add_config_value("django_next_version", "0.0", True)
    app.add_directive("versionadded", VersionDirective)
    app.add_directive("versionchanged", VersionDirective)
    app.connect("builder-inited", set_django_html_translator)
    app.add_node(
        ConsoleNode,
        html=(visit_console_html, None),
        latex=(visit_console_dummy, depart_console_dummy),
        man=(visit_console_dummy, depart_console_dummy),
        text=(visit_console_dummy, depart_console_dummy),
        texinfo=(visit_console_dummy, depart_console_dummy),
    )
    app.add_directive("console", ConsoleDirective)
    app.connect("html-page-context", html_page_context_hook)
    app.add_role("default-role-error", default_role_error)
    return {"parallel_read_safe": True}


def set_django_html_translator(app):
    if app.builder.format == "html":
        app.set_translator(app.builder.name, DjangoHTMLTranslator, override=True)


class VersionDirective(Directive):
    has_content = True
    required_arguments = 1
    optional_arguments = 1
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        if len(self.arguments) > 1:
            msg = """Only one argument accepted for directive '{directive_name}::'.
            Comments should be provided as content,
            not as an extra argument.""".format(directive_name=self.name)
            raise self.error(msg)

        env = self.state.document.settings.env
        ret = []
        node = addnodes.versionmodified()
        ret.append(node)

        if self.arguments[0] == env.config.django_next_version:
            node["version"] = "Development version"
        else:
            node["version"] = self.arguments[0]

        node["type"] = self.name
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        env.get_domain("changeset").note_changeset(node)
        return ret


class DjangoHTMLTranslator(HTMLTranslator):
    """
    Django-specific reST to HTML tweaks.
    """

    # Don't use border=1, which docutils does by default.
    def visit_table(self, node):
        self.context.append(self.compact_p)
        self.compact_p = True
        # Needed by Sphinx.
        self._table_row_indices.append(0)
        self.body.append(self.starttag(node, "table", CLASS="docutils"))

    def depart_table(self, node):
        self.compact_p = self.context.pop()
        self._table_row_indices.pop()
        self.body.append("</table>\n")

    def visit_desc_parameterlist(self, node):
        self.body.append("(")  # by default sphinx puts <big> around the "("
        self.optional_param_level = 0
        self.param_separator = node.child_text_separator
        # Counts 'parameter groups' being either a required parameter, or a set
        # of contiguous optional ones.
        required_params = [
            isinstance(c, addnodes.desc_parameter) for c in node.children
        ]
        # How many required parameters are left.
        self.required_params_left = sum(required_params)
        if sphinx_version < (7, 1):
            self.first_param = 1
        else:
            self.is_first_param = True
            self.params_left_at_level = 0
            self.param_group_index = 0
            self.list_is_required_param = required_params
            self.multi_line_parameter_list = False

    def depart_desc_parameterlist(self, node):
        self.body.append(")")

    #
    # Turn the "new in version" stuff (versionadded/versionchanged) into a
    # better callout -- the Sphinx default is just a little span,
    # which is a bit less obvious that I'd like.
    #
    # FIXME: these messages are all hardcoded in English. We need to change
    # that to accommodate other language docs, but I can't work out how to make
    # that work.
    #
    version_text = {
        "versionchanged": "Changed in Django %s",
        "versionadded": "New in Django %s",
    }

    def visit_versionmodified(self, node):
        self.body.append(self.starttag(node, "div", CLASS=node["type"]))
        version_text = self.version_text.get(node["type"])
        if version_text:
            title = "%s%s" % (version_text % node["version"], ":" if len(node) else ".")
            self.body.append('<span class="title">%s</span> ' % title)

    def depart_versionmodified(self, node):
        self.body.append("</div>\n")

    # Give each section a unique ID -- nice for custom CSS hooks
    def visit_section(self, node):
        old_ids = node.get("ids", [])
        node["ids"] = ["s-" + i for i in old_ids]
        node["ids"].extend(old_ids)
        super().visit_section(node)
        node["ids"] = old_ids


def parse_django_admin_node(env, sig, signode):
    command = sig.split(" ")[0]
    env.ref_context["std:program"] = command
    title = "django-admin %s" % sig
    signode += addnodes.desc_name(title, title)
    return command


class ConsoleNode(nodes.literal_block):
    """
    Custom node to override the visit/depart event handlers at registration
    time. Wrap a literal_block object and defer to it.
    """

    tagname = "ConsoleNode"

    def __init__(self, litblk_obj):
        self.wrapped = litblk_obj

    def __getattr__(self, attr):
        if attr == "wrapped":
            return self.__dict__.wrapped
        return getattr(self.wrapped, attr)


def visit_console_dummy(self, node):
    """Defer to the corresponding parent's handler."""
    self.visit_literal_block(node)


def depart_console_dummy(self, node):
    """Defer to the corresponding parent's handler."""
    self.depart_literal_block(node)


def visit_console_html(self, node):
    """Generate HTML for the console directive."""
    if self.builder.format == "html" and node["win_console_text"]:
        # Put a mark on the document object signaling the fact the directive
        # has been used on it.
        self.document._console_directive_used_flag = True
        uid = node["uid"]
        self.body.append("""\
<div class="console-block" id="console-block-%(id)s">
<input class="c-tab-unix" id="c-tab-%(id)s-unix" type="radio" name="console-%(id)s" \
checked>
<label for="c-tab-%(id)s-unix" title="Linux/macOS">&#xf17c/&#xf179</label>
<input class="c-tab-win" id="c-tab-%(id)s-win" type="radio" name="console-%(id)s">
<label for="c-tab-%(id)s-win" title="Windows">&#xf17a</label>
<section class="c-content-unix" id="c-content-%(id)s-unix">\n""" % {"id": uid})
        try:
            self.visit_literal_block(node)
        except nodes.SkipNode:
            pass
        self.body.append("</section>\n")

        self.body.append(
            '<section class="c-content-win" id="c-content-%(id)s-win">\n' % {"id": uid}
        )
        win_text = node["win_console_text"]
        highlight_args = {"force": True}
        linenos = node.get("linenos", False)

        def warner(msg):
            self.builder.warn(msg, (self.builder.current_docname, node.line))

        highlighted = self.highlighter.highlight_block(
            win_text, "doscon", warn=warner, linenos=linenos, **highlight_args
        )
        self.body.append(highlighted)
        self.body.append("</section>\n")
        self.body.append("</div>\n")
        raise nodes.SkipNode
    else:
        self.visit_literal_block(node)


class ConsoleDirective(CodeBlock):
    """
    A reStructuredText directive which renders a two-tab code block in which
    the second tab shows a Windows command line equivalent of the usual
    Unix-oriented examples.
    """

    required_arguments = 0
    # The 'doscon' Pygments formatter needs a prompt like this. '>' alone
    # won't do it because then it simply paints the whole command line as a
    # gray comment with no highlighting at all.
    WIN_PROMPT = r"...\> "

    def run(self):
        def args_to_win(cmdline):
            changed = False
            out = []
            for token in cmdline.split():
                if token[:2] == "./":
                    token = token[2:]
                    changed = True
                elif token[:2] == "~/":
                    token = "%HOMEPATH%\\" + token[2:]
                    changed = True
                elif token == "make":
                    token = "make.bat"
                    changed = True
                if "://" not in token and "git" not in cmdline:
                    out.append(token.replace("/", "\\"))
                    changed = True
                else:
                    out.append(token)
            if changed:
                return " ".join(out)
            return cmdline

        def cmdline_to_win(line):
            if line.startswith("# "):
                return "REM " + args_to_win(line[2:])
            if line.startswith("$ # "):
                return "REM " + args_to_win(line[4:])
            if line.startswith("$ ./manage.py"):
                return "manage.py " + args_to_win(line[13:])
            if line.startswith("$ manage.py"):
                return "manage.py " + args_to_win(line[11:])
            if line.startswith("$ ./runtests.py"):
                return "runtests.py " + args_to_win(line[15:])
            if line.startswith("$ ./"):
                return args_to_win(line[4:])
            if line.startswith("$ python3"):
                return "py " + args_to_win(line[9:])
            if line.startswith("$ python"):
                return "py " + args_to_win(line[8:])
            if line.startswith("$ "):
                return args_to_win(line[2:])
            return None

        def code_block_to_win(content):
            bchanged = False
            lines = []
            for line in content:
                modline = cmdline_to_win(line)
                if modline is None:
                    lines.append(line)
                else:
                    lines.append(self.WIN_PROMPT + modline)
                    bchanged = True
            if bchanged:
                return ViewList(lines)
            return None

        env = self.state.document.settings.env
        self.arguments = ["console"]
        lit_blk_obj = super().run()[0]

        # Only do work when an HTML-format Sphinx builder is being used,
        # invoke the default behavior for the rest.
        if env.app.builder.format != "html":
            return [lit_blk_obj]

        lit_blk_obj["uid"] = str(env.new_serialno("console"))
        # Only add the tabbed UI if there is actually a Windows-specific
        # version of the CLI example.
        win_content = code_block_to_win(self.content)
        if win_content is None:
            lit_blk_obj["win_console_text"] = None
        else:
            self.content = win_content
            lit_blk_obj["win_console_text"] = super().run()[0].rawsource

        # Replace the literal_node object returned by Sphinx's CodeBlock with
        # the ConsoleNode wrapper.
        return [ConsoleNode(lit_blk_obj)]


def html_page_context_hook(app, pagename, templatename, context, doctree):
    # Put a bool on the context used to render the template. It's used to
    # control inclusion of console-tabs.css. This way it's included only from
    # HTML files rendered from reST files where the ConsoleDirective is used.
    context["include_console_assets"] = getattr(
        doctree, "_console_directive_used_flag", False
    )


def default_role_error(
    name, rawtext, text, lineno, inliner, options=None, content=None
):
    msg = (
        "Default role used (`single backticks`): %s. Did you mean to use two "
        "backticks for ``code``, or miss an underscore for a `link`_ ?" % rawtext
    )
    logger.warning(msg, location=(inliner.document.current_source, lineno))
    return [nodes.Text(text)], []



from django import template

register = template.Library()


class AdminLogNode(template.Node):
    def __init__(self, limit, varname, user):
        self.limit = limit
        self.varname = varname
        self.user = user

    def __repr__(self):
        return "<GetAdminLog Node>"

    def render(self, context):
        entries = context["log_entries"]
        if self.user is not None:
            user_id = self.user
            if not user_id.isdigit():
                user_id = context[self.user].pk
            entries = entries.filter(user__pk=user_id)
        context[self.varname] = entries[: int(self.limit)]
        return ""


@register.tag
def get_admin_log(parser, token):
    """
    Populate a template variable with the admin log for the given criteria.

    Usage::

        {% get_admin_log [limit] as [varname] for_user [user_id_or_varname] %}

    Examples::

        {% get_admin_log 10 as admin_log for_user 23 %}
        {% get_admin_log 10 as admin_log for_user user %}
        {% get_admin_log 10 as admin_log %}

    Note that ``user_id_or_varname`` can be a hard-coded integer (user ID)
    or the name of a template context variable containing the user object
    whose ID you want.
    """
    tokens = token.contents.split()
    if len(tokens) < 4:
        raise template.TemplateSyntaxError(
            "'get_admin_log' statements require two arguments"
        )
    if not tokens[1].isdigit():
        raise template.TemplateSyntaxError(
            "First argument to 'get_admin_log' must be an integer"
        )
    if tokens[2] != "as":
        raise template.TemplateSyntaxError(
            "Second argument to 'get_admin_log' must be 'as'"
        )
    if len(tokens) > 4:
        if tokens[4] != "for_user":
            raise template.TemplateSyntaxError(
                "Fourth argument to 'get_admin_log' must be 'for_user'"
            )
    return AdminLogNode(
        limit=tokens[1],
        varname=tokens[3],
        user=(tokens[5] if len(tokens) > 5 else None),
    )



"""Base email backend class."""

from django.core.mail import InvalidMailer
from django.utils.deprecation import RemovedInDjango70Warning, warn_about_external_use

# RemovedInDjango70Warning.
_NOT_PROVIDED = object()


class BaseEmailBackend:
    """
    Base class for email backend implementations.

    Subclasses must implement at least send_messages().

    Subclass __init__() should pass alias and unknown **kwargs to
    super().__init__() for proper error reporting.

    open() and close() can be called indirectly by using a backend object as a
    context manager:

       with backend as connection:
           # do something with connection
           pass
    """

    # RemovedInDjango70Warning: fail_silently, _ignore_unknown_kwargs.
    def __init__(
        self,
        fail_silently=_NOT_PROVIDED,
        *,
        alias=None,
        _ignore_unknown_kwargs=None,
        **kwargs,
    ):
        self.alias = alias

        # RemovedInDjango70Warning.
        if fail_silently is _NOT_PROVIDED:
            self._fail_silently = False
        else:
            self._fail_silently = fail_silently
            # Force deprecation warning unless in _ignore_unknown_kwargs.
            kwargs["fail_silently"] = fail_silently
        if _ignore_unknown_kwargs:
            for ignored in _ignore_unknown_kwargs:
                kwargs.pop(ignored, None)

        if kwargs:
            kwarg_names = ", ".join(repr(key) for key in kwargs.keys())

            # RemovedInDjango70Warning.
            if alias is None:
                # Not being initialized from mail.mailers. Unknown kwargs are
                # ignored -- with a deprecation warning if they originated from
                # outside Django (either direct construction of a built-in
                # backend or a super.__init__() call from a custom subclass).
                # Use something more precise than self.__class__.__name__,
                # which is often just "EmailBackend".
                class_name = f"{type(self).__module__}.{type(self).__qualname__}"
                warn_about_external_use(
                    f"{class_name}.__init__() does not support {kwarg_names}. "
                    "In Django 7.0, BaseEmailBackend will raise a TypeError "
                    "for unknown keyword arguments.",
                    RemovedInDjango70Warning,
                    skip_name_prefixes="django.core.mail.backends",
                )
                return

            raise InvalidMailer(f"Unknown options {kwarg_names}.", alias=alias)

    # RemovedInDjango70Warning.
    def __getattr__(self, name):
        if name == "fail_silently":
            msg = (
                "BaseEmailBackend.fail_silently is deprecated. A subclass "
                "that supports fail_silently must set its own attribute."
            )
            warn_about_external_use(msg, RemovedInDjango70Warning)
            return self._fail_silently
        raise AttributeError(
            f"{type(self).__name__!r} object has no attribute {name!r}"
        )

    def open(self):
        """
        Open a network connection.

        This method can be overwritten by backend implementations to
        open a network connection.

        It's up to the backend implementation to track the status of
        a network connection if it's needed by the backend.

        This method can be called by applications to force a single
        network connection to be used when sending mails. See the
        send_messages() method of the SMTP backend for a reference
        implementation.

        The default implementation does nothing.
        """
        pass

    def close(self):
        """Close a network connection."""
        pass

    def __enter__(self):
        try:
            self.open()
        except Exception:
            self.close()
            raise
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def send_messages(self, email_messages):
        """
        Send one or more EmailMessage objects and return the number of email
        messages sent.
        """
        raise NotImplementedError(
            "subclasses of BaseEmailBackend must override send_messages() method"
        )



from django.forms import Media
from django.forms.widgets import Script
from django.template import Context, Template
from django.test import SimpleTestCase, override_settings


@override_settings(STATIC_URL="/static/")
class CspNonceTagTests(SimpleTestCase):
    def test_with_nonce_in_context(self):
        t = Template("<script {% csp_nonce_attr %}></script>")
        result = t.render(Context({"csp_nonce": "abc123"}))
        self.assertEqual(result, '<script nonce="abc123"></script>')

    def test_without_csp_nonce_in_context(self):
        t = Template("<script {% csp_nonce_attr %}></script>")
        result = t.render(Context())
        self.assertEqual(result, "<script ></script>")

    def test_with_csp_nonce_none(self):
        t = Template("<script {% csp_nonce_attr %}></script>")
        result = t.render(Context({"csp_nonce": None}))
        self.assertEqual(result, "<script ></script>")

    def test_nonce_is_escaped(self):
        t = Template("<script {% csp_nonce_attr %}></script>")
        result = t.render(Context({"csp_nonce": '<script>"'}))
        self.assertIn("&lt;", result)
        self.assertNotIn("<script>", result)


@override_settings(STATIC_URL="/static/")
class CspNonceTagWithMediaTests(SimpleTestCase):
    def test_with_nonce_in_context(self):
        media = Media(js=["/path/to/js"])
        t = Template("{% csp_nonce_attr media %}")
        result = t.render(Context({"media": media, "csp_nonce": "abc123"}))
        self.assertHTMLEqual(
            result,
            '<script src="/path/to/js" nonce="abc123"></script>',
        )

    def test_without_csp_nonce_in_context(self):
        media = Media(js=["/path/to/js"])
        t = Template("{% csp_nonce_attr media %}")
        result = t.render(Context({"media": media}))
        self.assertHTMLEqual(result, '<script src="/path/to/js"></script>')

    def test_with_csp_nonce_none(self):
        media = Media(js=["/path/to/js"])
        t = Template("{% csp_nonce_attr media %}")
        result = t.render(Context({"media": media, "csp_nonce": None}))
        self.assertHTMLEqual(result, '<script src="/path/to/js"></script>')

    def test_css_and_js(self):
        media = Media(
            css={"all": ["/path/to/css"]},
            js=["/path/to/js"],
        )
        t = Template("{% csp_nonce_attr media %}")
        result = t.render(Context({"media": media, "csp_nonce": "abc123"}))
        self.assertHTMLEqual(
            result,
            '<link href="/path/to/css" media="all" nonce="abc123" rel="stylesheet">\n'
            '<script src="/path/to/js" nonce="abc123"></script>',
        )

    def test_with_script_object(self):
        media = Media(js=[Script("/path/to/js", integrity="sha256-abc")])
        t = Template("{% csp_nonce_attr media %}")
        result = t.render(Context({"media": media, "csp_nonce": "abc123"}))
        self.assertHTMLEqual(
            result,
            '<script src="/path/to/js" integrity="sha256-abc"'
            ' nonce="abc123"></script>',
        )

    def test_output_is_safe(self):
        media = Media(js=["/path/to/js"])
        t = Template("{% csp_nonce_attr media %}")
        result = t.render(Context({"media": media, "csp_nonce": "abc123"}))
        self.assertIn("<script", result)
        self.assertNotIn("&lt;", result)

    def test_script_with_conflicting_nonce_raises(self):
        media = Media(js=[Script("/path/to/js", nonce="static")])
        t = Template("{% csp_nonce_attr media %}")
        msg = "Script has conflicting attributes: nonce"
        with self.assertRaisesMessage(ValueError, msg):
            t.render(Context({"media": media, "csp_nonce": "abc123"}))



import inspect
import sys
import warnings
from contextlib import contextmanager

from django.test import SimpleTestCase
from django.utils.deprecation import warn_about_implementation
from django.views.decorators import csrf


class WarnAboutImplementationTests(SimpleTestCase):
    @contextmanager
    def assertWarnsAboutLine(self, category, message, *, offset):
        caller_frame = inspect.currentframe().f_back.f_back
        with self.assertWarnsMessage(category, message) as warning:
            yield
        self.assertEqual(warning.filename, caller_frame.f_code.co_filename)
        self.assertEqual(warning.lineno, caller_frame.f_lineno + offset)

    def test_function(self):
        def some_function():
            pass

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-3):
            warn_about_implementation("deprecated", Warning, some_function)

    def test_class(self):
        class SomeClass:
            pass

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-3):
            warn_about_implementation("deprecated", Warning, SomeClass)

    def test_class_init(self):
        class SomeClass:
            def __init__(self):
                pass

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-3):
            warn_about_implementation("deprecated", Warning, SomeClass.__init__)

    def test_method(self):
        class SomeClass:
            def some_method(self):
                pass

        for case, target in [
            ("unbound", SomeClass.some_method),
            ("bound", SomeClass().some_method),
        ]:
            with (
                self.subTest(case),
                self.assertWarnsAboutLine(Warning, "deprecated", offset=-9),
            ):
                warn_about_implementation("deprecated", Warning, target)

    def test_subclass_method(self):
        class BaseClass:
            def some_method(self):
                pass

            def issue_warning(self):
                warn_about_implementation("deprecated", Warning, self.some_method)

        class SubClass(BaseClass):
            def some_method(self):
                pass

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-3):
            SubClass().issue_warning()

    def test_my_own_method(self):
        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-1):
            warn_about_implementation("deprecated", Warning, self.test_my_own_method)

    def test_classmethod(self):
        class SomeClass:
            @classmethod
            def some_classmethod(cls):
                pass

        for case, target in [
            ("unbound", SomeClass.some_classmethod),
            ("bound", SomeClass().some_classmethod),
        ]:
            with (
                self.subTest(case),
                self.assertWarnsAboutLine(Warning, "deprecated", offset=-10),
            ):
                warn_about_implementation("deprecated", Warning, target)

    def test_staticmethod(self):
        class SomeClass:
            @staticmethod
            def some_staticmethod():
                pass

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-4):
            warn_about_implementation(
                "deprecated", Warning, SomeClass.some_staticmethod
            )

    def test_property(self):
        class SomeClass:
            @property
            def some_property(self):
                return None

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-4):
            warn_about_implementation(
                "deprecated",
                Warning,
                inspect.getattr_static(SomeClass(), "some_property"),
            )

    def test_decorated_function(self):
        @csrf.csrf_exempt
        @csrf.ensure_csrf_cookie
        def some_view(request):
            pass

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-5):
            warn_about_implementation("deprecated", Warning, some_view)

    def test_decorated_method(self):
        class SomeClass:
            @csrf.csrf_exempt
            @csrf.ensure_csrf_cookie
            def some_method(self):
                pass

        for case, target in [
            ("unbound", SomeClass.some_method),
            ("bound", SomeClass().some_method),
        ]:
            with (
                self.subTest(case),
                self.assertWarnsAboutLine(Warning, "deprecated", offset=-11),
            ):
                warn_about_implementation("deprecated", Warning, target)

    def test_decorated_staticmethod(self):
        class SomeClass:
            @staticmethod
            @csrf.csrf_exempt
            @csrf.ensure_csrf_cookie
            def some_staticmethod():
                pass

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-6):
            warn_about_implementation(
                "deprecated", Warning, SomeClass.some_staticmethod
            )

    def test_no_source_file(self):
        # When the source file can't be determined, falls back to warning about
        # the caller. (Builtins like `dict` have no source file.)
        with self.assertWarnsAboutLine(Warning, "deprecated", offset=1):
            warn_about_implementation("deprecated", Warning, dict)

    def test_rejects_invalid_target(self):
        msg = (
            "target must be a function, class, bound method, or unbound descriptor "
            "(classmethod, staticmethod, or property)."
        )
        for invalid in ["string", object(), 123, None]:
            with self.subTest(target=invalid), self.assertRaisesMessage(TypeError, msg):
                warn_about_implementation("deprecated", Warning, invalid)

    def test_respects_warning_registry(self):
        def some_function():
            pass

        # The "default" action only works correctly when warn_explicit() is
        # called with the correct module and warning registry. (This test must
        # use "default" or "module", not "once". CPython has an undocumented
        # optimization for "once" that uses a global registry.)
        with warnings.catch_warnings(record=True, action="default") as captured:
            warn_about_implementation("deprecated", Warning, some_function)
            self.assertEqual(len(captured), 1)

            captured.clear()
            warn_about_implementation("deprecated", Warning, some_function)
            self.assertEqual(len(captured), 0)

    def test_creates_warning_registry_if_needed(self):
        def some_function():
            pass

        module = sys.modules[some_function.__module__]
        if hasattr(module, "__warningregistry__"):
            old_registry = module.__warningregistry__
            del module.__warningregistry__
        else:
            old_registry = None

        try:
            with warnings.catch_warnings(record=True, action="default") as captured:
                warn_about_implementation("deprecated", Warning, some_function)
                warn_about_implementation("deprecated", Warning, some_function)

            self.assertEqual(len(captured), 1)
            self.assertIsInstance(module.__warningregistry__, dict)
            self.assertNotEqual(module.__warningregistry__, {})
        finally:
            if old_registry is not None:
                module.__warningregistry__ = old_registry
            elif hasattr(module, "__warningregistry__"):
                del module.__warningregistry__

    def test_missing_module(self):
        def some_function():
            pass

        some_function.__module__ = "not_a_module"

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=-5):
            warn_about_implementation("deprecated", Warning, some_function)

    def test_non_standard_module(self):
        sys.modules["sealed_module"] = type("SealedModule", (), {"__slots__": ()})()
        self.addCleanup(sys.modules.pop, "sealed_module")

        def some_function():
            pass

        some_function.__module__ = "sealed_module"

        with self.assertWarnsAboutLine(Warning, "deprecated", offset=2):
            # Module is not inspectable, so the warning is issued generically.
            warn_about_implementation("deprecated", Warning, some_function)



from django.db import migrations, models


def add_book(apps, schema_editor):
    apps.get_model("migration_test_data_persistence", "Book").objects.using(
        schema_editor.connection.alias,
    ).create(
        title="I Love Django",
    )


class Migration(migrations.Migration):
    dependencies = [("migration_test_data_persistence", "0001_initial")]

    operations = [
        migrations.RunPython(
            add_book,
        ),
        migrations.CreateModel(
            name="Unmanaged",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=100)),
            ],
            options={
                "managed": False,
            },
        ),
        migrations.AlterField(
            model_name="book",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



import gc

from django.db.models.base import ModelState, ModelStateFieldsCacheDescriptor
from django.test import SimpleTestCase
from django.test.utils import garbage_collect

from .models import Worker, WorkerProfile


class ModelStateTests(SimpleTestCase):
    def test_fields_cache_descriptor(self):
        self.assertIsInstance(ModelState.fields_cache, ModelStateFieldsCacheDescriptor)

    def test_one_to_one_field_cycle_collection(self):
        self.addCleanup(gc.set_debug, gc.get_debug())
        gc.set_debug(gc.DEBUG_SAVEALL)

        def clear_garbage():
            del gc.garbage[:]

        self.addCleanup(clear_garbage)

        worker = Worker()
        profile = WorkerProfile(worker=worker)
        worker_id = id(worker)

        del worker
        del profile

        garbage_collect()

        leaked = [obj for obj in gc.garbage if id(obj) == worker_id]
        self.assertEqual(leaked, [])



"""SMTP email backend class."""

import email.policy
import smtplib
import ssl
import threading
from email.headerregistry import Address, AddressHeader

from django.conf import settings
from django.core.mail import InvalidMailer
from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.utils import DNS_NAME
from django.utils.deprecation import RemovedInDjango70Warning, warn_about_external_use
from django.utils.encoding import force_str, punycode
from django.utils.functional import cached_property


class EmailBackend(BaseEmailBackend):
    """
    A wrapper that manages the SMTP network connection.
    """

    def __init__(
        self,
        host=None,
        port=None,
        username=None,
        password=None,
        use_tls=None,
        fail_silently=False,
        use_ssl=None,
        timeout=None,
        ssl_keyfile=None,
        ssl_certfile=None,
        **kwargs,
    ):
        # RemovedInDjango70Warning.
        if "alias" not in kwargs:
            msg = (
                "Directly creating EmailBackend instances is deprecated. "
                "Use mail.mailers instead."
            )
            warn_about_external_use(msg, RemovedInDjango70Warning)

        super().__init__(**kwargs)
        self.fail_silently = fail_silently
        self.connection = None
        self._partial_connection = None
        self._lock = threading.RLock()

        # RemovedInDjango70Warning.
        if self.alias is None:
            # Use deprecated settings when MAILERS not enabled.
            self.host = host or settings.EMAIL_HOST
            self.port = port or settings.EMAIL_PORT
            self.username = settings.EMAIL_HOST_USER if username is None else username
            self.password = (
                settings.EMAIL_HOST_PASSWORD if password is None else password
            )
            self.use_tls = settings.EMAIL_USE_TLS if use_tls is None else use_tls
            self.use_ssl = settings.EMAIL_USE_SSL if use_ssl is None else use_ssl
            self.timeout = settings.EMAIL_TIMEOUT if timeout is None else timeout
            self.ssl_keyfile = (
                settings.EMAIL_SSL_KEYFILE if ssl_keyfile is None else ssl_keyfile
            )
            self.ssl_certfile = (
                settings.EMAIL_SSL_CERTFILE if ssl_certfile is None else ssl_certfile
            )
            if self.use_ssl and self.use_tls:
                raise ValueError(
                    "EMAIL_USE_TLS/EMAIL_USE_SSL are mutually exclusive, so "
                    "only set one of those settings to True."
                )
            return

        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.use_tls = use_tls if use_tls is not None else False
        self.use_ssl = use_ssl if use_ssl is not None else False
        self.timeout = timeout
        self.ssl_keyfile = ssl_keyfile
        self.ssl_certfile = ssl_certfile
        if self.host is None:
            raise InvalidMailer("OPTIONS must define 'host'.", alias=self.alias)
        if self.use_ssl and self.use_tls:
            raise InvalidMailer(
                "The 'use_ssl' and 'use_tls' OPTIONS are incompatible. "
                "Set at most one of them to True.",
                alias=self.alias,
            )
        if self.port is None:
            if self.use_ssl:
                self.port = 465
            elif self.use_tls:
                self.port = 587
            else:
                self.port = 25

    @property
    def connection_class(self):
        return smtplib.SMTP_SSL if self.use_ssl else smtplib.SMTP

    @cached_property
    def ssl_context(self):
        if self.ssl_certfile or self.ssl_keyfile:
            ssl_context = ssl.SSLContext(protocol=ssl.PROTOCOL_TLS_CLIENT)
            ssl_context.load_cert_chain(self.ssl_certfile, self.ssl_keyfile)
            return ssl_context
        else:
            return ssl.create_default_context()

    def open(self):
        """
        Ensure an open connection to the email server. Return whether or not a
        new connection was required (True or False) or None if an exception
        passed silently.
        """
        if self.connection:
            # Nothing to do if the connection is already open.
            return False

        # If a connection was partially opened before, close it.
        if self._partial_connection is not None:
            self._close_connection(self._partial_connection)
            self._partial_connection = None

        # If local_hostname is not specified, socket.getfqdn() gets used.
        # For performance, we use the cached FQDN for local_hostname.
        connection_params = {"local_hostname": DNS_NAME.get_fqdn()}
        if self.timeout is not None:
            connection_params["timeout"] = self.timeout
        if self.use_ssl:
            connection_params["context"] = self.ssl_context
        try:
            self._partial_connection = self.connection_class(
                self.host, self.port, **connection_params
            )

            # TLS/SSL are mutually exclusive, so only attempt TLS over
            # non-secure connections.
            if not self.use_ssl and self.use_tls:
                self._partial_connection.starttls(context=self.ssl_context)
            if self.username and self.password:
                self._partial_connection.login(self.username, self.password)

            # Don't set connection until it's fully configured.
            self.connection = self._partial_connection
            self._partial_connection = None

            return True
        except OSError:
            if not self.fail_silently:
                raise

    def _close_connection(self, connection):
        try:
            connection.quit()
        except (ssl.SSLError, smtplib.SMTPServerDisconnected):
            # This happens when calling quit() on a TLS connection
            # sometimes, or when the connection was already disconnected
            # by the server.
            connection.close()
        except smtplib.SMTPException:
            if self.fail_silently:
                return
            raise

    def close(self):
        """Close the connection to the email server."""
        if self._partial_connection is not None:
            try:
                self._close_connection(self._partial_connection)
            finally:
                self._partial_connection = None
        if self.connection is not None:
            try:
                self._close_connection(self.connection)
            finally:
                self.connection = None

    def send_messages(self, email_messages):
        """
        Send one or more EmailMessage objects and return the number of email
        messages sent.
        """
        if not email_messages:
            return 0
        with self._lock:
            new_conn_created = self.open()
            if not self.connection or new_conn_created is None:
                # We failed silently on open().
                # Trying to send would be pointless.
                return 0
            num_sent = 0
            try:
                for message in email_messages:
                    sent = self._send(message)
                    if sent:
                        num_sent += 1
            finally:
                if new_conn_created:
                    self.close()
        return num_sent

    def _send(self, email_message):
        """A helper method that does the actual sending."""
        if not email_message.recipients():
            return False
        from_email = self.prep_address(email_message.from_email)
        recipients = [self.prep_address(addr) for addr in email_message.recipients()]
        message = email_message.message(policy=email.policy.SMTP)
        try:
            self.connection.sendmail(from_email, recipients, message.as_bytes())
        except smtplib.SMTPException:
            if not self.fail_silently:
                raise
            return False
        return True

    def prep_address(self, address, force_ascii=True):
        """
        Return the addr-spec portion of an email address. Raises ValueError for
        invalid addresses, including CR/NL injection.

        If force_ascii is True, apply IDNA encoding to non-ASCII domains, and
        raise ValueError for non-ASCII local-parts (which can't be encoded).
        Otherwise, leave Unicode characters unencoded (e.g., for sending with
        SMTPUTF8).
        """
        address = force_str(address)
        parsed = AddressHeader.value_parser(address)
        defects = set(str(defect) for defect in parsed.all_defects)
        # Django allows local mailboxes like "From: webmaster" (#15042).
        defects.discard("addr-spec local part with no domain")
        if not force_ascii:
            # PY315: Non-ASCII local-part is valid with SMTPUTF8. This check
            # can be removed once the minimum supported Python version is 3.15
            # (so the fix for https://github.com/python/cpython/issues/81074
            # is in all supported versions).
            defects.discard("local-part contains non-ASCII characters)")
        if defects:
            raise ValueError(f"Invalid address {address!r}: {'; '.join(defects)}")

        mailboxes = parsed.all_mailboxes
        if len(mailboxes) != 1:
            raise ValueError(f"Invalid address {address!r}: must be a single address")

        mailbox = mailboxes[0]
        if force_ascii and mailbox.domain and not mailbox.domain.isascii():
            # Re-compose an addr-spec with the IDNA encoded domain.
            domain = punycode(mailbox.domain)
            return str(Address(username=mailbox.local_part, domain=domain))
        else:
            return mailbox.addr_spec



import datetime
import decimal
import sqlite3
import uuid
from functools import lru_cache
from itertools import chain

from django.conf import settings
from django.core.exceptions import FieldError
from django.db import DatabaseError, NotSupportedError, models
from django.db.backends.base.operations import BaseDatabaseOperations
from django.db.models.constants import OnConflict
from django.db.models.expressions import Col
from django.utils import timezone
from django.utils.dateparse import parse_date, parse_datetime, parse_time
from django.utils.functional import cached_property

from .base import Database

UNSUPPORTED_DATETIME_AGGREGATES = (
    models.Sum,
    models.Avg,
    models.Variance,
    models.StdDev,
)
DATETIME_FIELDS = (models.DateField, models.DateTimeField, models.TimeField)


class DatabaseOperations(BaseDatabaseOperations):
    cast_char_field_without_max_length = "text"
    cast_data_types = {
        "DateField": "TEXT",
        "DateTimeField": "TEXT",
    }
    explain_prefix = "EXPLAIN QUERY PLAN"
    # List of datatypes to that cannot be extracted with JSON_EXTRACT() on
    # SQLite. Use JSON_TYPE() instead.
    jsonfield_datatype_values = frozenset(["null", "false", "true"])

    def check_expression_support(self, expression):
        if isinstance(expression, UNSUPPORTED_DATETIME_AGGREGATES):
            for expr in expression.get_source_expressions():
                try:
                    output_field = expr.output_field
                except (AttributeError, FieldError):
                    # Not every subexpression has an output_field which is fine
                    # to ignore.
                    pass
                else:
                    if isinstance(output_field, DATETIME_FIELDS):
                        klass = expression.__class__.__name__
                        raise NotSupportedError(
                            f"SQLite does not support {klass} on date or time "
                            "fields, because they are stored as text."
                        )
        if (
            isinstance(expression, models.Aggregate)
            and expression.distinct
            and len(expression.source_expressions) > 1
        ):
            raise NotSupportedError(
                "SQLite doesn't support DISTINCT on aggregate functions "
                "accepting multiple arguments."
            )

    def date_extract_sql(self, lookup_type, sql, params):
        """
        Support EXTRACT with a user-defined function django_date_extract()
        that's registered in connect(). Use single quotes because this is a
        string and could otherwise cause a collision with a field name.
        """
        return f"django_date_extract(%s, {sql})", (lookup_type.lower(), *params)

    def format_for_duration_arithmetic(self, sql):
        """Do nothing since formatting is handled in the custom function."""
        return sql

    def date_trunc_sql(self, lookup_type, sql, params, tzname=None):
        return f"django_date_trunc(%s, {sql}, %s, %s)", (
            lookup_type.lower(),
            *params,
            *self._convert_tznames_to_sql(tzname),
        )

    def time_trunc_sql(self, lookup_type, sql, params, tzname=None):
        return f"django_time_trunc(%s, {sql}, %s, %s)", (
            lookup_type.lower(),
            *params,
            *self._convert_tznames_to_sql(tzname),
        )

    def _convert_tznames_to_sql(self, tzname):
        if tzname and settings.USE_TZ:
            return tzname, self.connection.timezone_name
        return None, None

    def datetime_cast_date_sql(self, sql, params, tzname):
        return f"django_datetime_cast_date({sql}, %s, %s)", (
            *params,
            *self._convert_tznames_to_sql(tzname),
        )

    def datetime_cast_time_sql(self, sql, params, tzname):
        return f"django_datetime_cast_time({sql}, %s, %s)", (
            *params,
            *self._convert_tznames_to_sql(tzname),
        )

    def datetime_extract_sql(self, lookup_type, sql, params, tzname):
        return f"django_datetime_extract(%s, {sql}, %s, %s)", (
            lookup_type.lower(),
            *params,
            *self._convert_tznames_to_sql(tzname),
        )

    def datetime_trunc_sql(self, lookup_type, sql, params, tzname):
        return f"django_datetime_trunc(%s, {sql}, %s, %s)", (
            lookup_type.lower(),
            *params,
            *self._convert_tznames_to_sql(tzname),
        )

    def time_extract_sql(self, lookup_type, sql, params):
        return f"django_time_extract(%s, {sql})", (lookup_type.lower(), *params)

    def pk_default_value(self):
        return "NULL"

    def _quote_params_for_last_executed_query(self, params):
        """
        Only for last_executed_query! Don't use this to execute SQL queries!
        """
        connection = self.connection.connection
        variable_limit = self.connection.features.max_query_params
        column_limit = connection.getlimit(sqlite3.SQLITE_LIMIT_COLUMN)
        batch_size = min(variable_limit, column_limit)

        if len(params) > batch_size:
            results = ()
            for index in range(0, len(params), batch_size):
                chunk = params[index : index + batch_size]
                results += self._quote_params_for_last_executed_query(chunk)
            return results

        sql = "SELECT " + ", ".join(["QUOTE(?)"] * len(params))
        # Bypass Django's wrappers and use the underlying sqlite3 connection
        # to avoid logging this query - it would trigger infinite recursion.
        cursor = self.connection.connection.cursor()
        # Native sqlite3 cursors cannot be used as context managers.
        try:
            return cursor.execute(sql, params).fetchone()
        finally:
            cursor.close()

    def last_executed_query(self, cursor, sql, params):
        # Python substitutes parameters in Modules/_sqlite/cursor.c with:
        # bind_parameters(state, self->statement, parameters);
        # Unfortunately there is no way to reach self->statement from Python,
        # so we quote and substitute parameters manually.
        if params:
            if isinstance(params, (list, tuple)):
                params = self._quote_params_for_last_executed_query(params)
            else:
                values = tuple(params.values())
                values = self._quote_params_for_last_executed_query(values)
                params = dict(zip(params, values))
            return sql % params
        # For consistency with SQLiteCursorWrapper.execute(), just return sql
        # when there are no parameters. See #13648 and #17158.
        else:
            return sql

    def quote_name(self, name):
        if name.startswith('"') and name.endswith('"'):
            return name  # Quoting once is enough.
        return '"%s"' % name

    def no_limit_value(self):
        return -1

    def __references_graph(self, table_name):
        query = """
        WITH tables AS (
            SELECT %s name
            UNION
            SELECT sqlite_master.name
            FROM sqlite_master
            JOIN tables ON (sql REGEXP %s || tables.name || %s)
        ) SELECT name FROM tables;
        """
        params = (
            table_name,
            r'(?i)\s+references\s+("|\')?',
            r'("|\')?\s*\(',
        )
        with self.connection.cursor() as cursor:
            results = cursor.execute(query, params)
            return [row[0] for row in results.fetchall()]

    @cached_property
    def _references_graph(self):
        # 512 is large enough to fit the ~330 tables (as of this writing) in
        # Django's test suite.
        return lru_cache(maxsize=512)(self.__references_graph)

    def sql_flush(self, style, tables, *, reset_sequences=False, allow_cascade=False):
        if tables and allow_cascade:
            # Simulate TRUNCATE CASCADE by recursively collecting the tables
            # referencing the tables to be flushed.
            tables = set(
                chain.from_iterable(self._references_graph(table) for table in tables)
            )
        sql = [
            "%s %s %s;"
            % (
                style.SQL_KEYWORD("DELETE"),
                style.SQL_KEYWORD("FROM"),
                style.SQL_FIELD(self.quote_name(table)),
            )
            for table in tables
        ]
        if reset_sequences:
            sequences = [{"table": table} for table in tables]
            sql.extend(self.sequence_reset_by_name_sql(style, sequences))
        return sql

    def sequence_reset_by_name_sql(self, style, sequences):
        if not sequences:
            return []
        return [
            "%s %s %s %s = 0 %s %s %s (%s);"
            % (
                style.SQL_KEYWORD("UPDATE"),
                style.SQL_TABLE(self.quote_name("sqlite_sequence")),
                style.SQL_KEYWORD("SET"),
                style.SQL_FIELD(self.quote_name("seq")),
                style.SQL_KEYWORD("WHERE"),
                style.SQL_FIELD(self.quote_name("name")),
                style.SQL_KEYWORD("IN"),
                ", ".join(
                    ["'%s'" % sequence_info["table"] for sequence_info in sequences]
                ),
            ),
        ]

    def adapt_datetimefield_value(self, value):
        if value is None:
            return None

        # SQLite doesn't support tz-aware datetimes
        if timezone.is_aware(value):
            if settings.USE_TZ:
                value = timezone.make_naive(value, self.connection.timezone)
            else:
                raise ValueError(
                    "SQLite backend does not support timezone-aware datetimes when "
                    "USE_TZ is False."
                )

        return str(value)

    def adapt_timefield_value(self, value):
        if value is None:
            return None

        # SQLite doesn't support tz-aware datetimes
        if timezone.is_aware(value):
            raise ValueError("SQLite backend does not support timezone-aware times.")

        return str(value)

    def get_db_converters(self, expression):
        converters = super().get_db_converters(expression)
        internal_type = expression.output_field.get_internal_type()
        if internal_type == "DateTimeField":
            converters.append(self.convert_datetimefield_value)
        elif internal_type == "DateField":
            converters.append(self.convert_datefield_value)
        elif internal_type == "TimeField":
            converters.append(self.convert_timefield_value)
        elif internal_type == "DecimalField":
            converters.append(self.get_decimalfield_converter(expression))
        elif internal_type == "UUIDField":
            converters.append(self.convert_uuidfield_value)
        elif internal_type == "BooleanField":
            converters.append(self.convert_booleanfield_value)
        return converters

    def convert_datetimefield_value(self, value, expression, connection):
        if value is not None:
            if not isinstance(value, datetime.datetime):
                value = parse_datetime(value)
            if settings.USE_TZ and not timezone.is_aware(value):
                value = timezone.make_aware(value, self.connection.timezone)
        return value

    def convert_datefield_value(self, value, expression, connection):
        if value is not None:
            if not isinstance(value, datetime.date):
                value = parse_date(value)
        return value

    def convert_timefield_value(self, value, expression, connection):
        if value is not None:
            if not isinstance(value, datetime.time):
                value = parse_time(value)
        return value

    @staticmethod
    def _create_decimal(value):
        if isinstance(value, (int, str)):
            return decimal.Decimal(value)
        return decimal.Context(prec=15).create_decimal_from_float(value)

    def get_decimalfield_converter(self, expression):
        # SQLite stores only 15 significant digits. Digits coming from
        # float inaccuracy must be removed.
        if isinstance(expression, Col):
            quantize_value = decimal.Decimal(1).scaleb(
                -expression.output_field.decimal_places
            )

            def converter(value, expression, connection):
                if value is not None:
                    return self._create_decimal(value).quantize(
                        quantize_value, context=expression.output_field.context
                    )

        else:

            def converter(value, expression, connection):
                if value is not None:
                    return self._create_decimal(value)

        return converter

    def convert_uuidfield_value(self, value, expression, connection):
        if value is not None:
            value = uuid.UUID(value)
        return value

    def convert_booleanfield_value(self, value, expression, connection):
        return bool(value) if value in (1, 0) else value

    def combine_expression(self, connector, sub_expressions):
        # SQLite doesn't have a ^ operator, so use the user-defined POWER
        # function that's registered in connect().
        if connector == "^":
            return "POWER(%s)" % ",".join(sub_expressions)
        elif connector == "#":
            return "BITXOR(%s)" % ",".join(sub_expressions)
        return super().combine_expression(connector, sub_expressions)

    def combine_duration_expression(self, connector, sub_expressions):
        if connector not in ["+", "-", "*", "/"]:
            raise DatabaseError("Invalid connector for timedelta: %s." % connector)
        fn_params = ["'%s'" % connector, *sub_expressions]
        if len(fn_params) > 3:
            raise ValueError("Too many params for timedelta operations.")
        return "django_format_dtdelta(%s)" % ", ".join(fn_params)

    def integer_field_range(self, internal_type):
        # SQLite doesn't enforce any integer constraints, but sqlite3 supports
        # integers up to 64 bits.
        if internal_type in [
            "PositiveBigIntegerField",
            "PositiveIntegerField",
            "PositiveSmallIntegerField",
        ]:
            return (0, 9223372036854775807)
        return (-9223372036854775808, 9223372036854775807)

    def subtract_temporals(self, internal_type, lhs, rhs):
        lhs_sql, lhs_params = lhs
        rhs_sql, rhs_params = rhs
        params = (*lhs_params, *rhs_params)
        if internal_type == "TimeField":
            return "django_time_diff(%s, %s)" % (lhs_sql, rhs_sql), params
        return "django_timestamp_diff(%s, %s)" % (lhs_sql, rhs_sql), params

    def insert_statement(self, on_conflict=None):
        if on_conflict == OnConflict.IGNORE:
            return "INSERT OR IGNORE INTO"
        return super().insert_statement(on_conflict=on_conflict)

    def on_conflict_suffix_sql(self, fields, on_conflict, update_fields, unique_fields):
        if (
            on_conflict == OnConflict.UPDATE
            and self.connection.features.supports_update_conflicts_with_target
        ):
            return "ON CONFLICT(%s) DO UPDATE SET %s" % (
                ", ".join(map(self.quote_name, unique_fields)),
                ", ".join(
                    [
                        f"{field} = EXCLUDED.{field}"
                        for field in map(self.quote_name, update_fields)
                    ]
                ),
            )
        return super().on_conflict_suffix_sql(
            fields,
            on_conflict,
            update_fields,
            unique_fields,
        )

    def force_group_by(self):
        return ["GROUP BY TRUE"] if Database.sqlite_version_info < (3, 39) else []

    def format_json_path_numeric_index(self, num):
        return "[#%s]" % num if num < 0 else super().format_json_path_numeric_index(num)



"""
Backend for test environment.
"""

import copy

from django.core import mail
from django.core.mail.backends.base import BaseEmailBackend


class EmailBackend(BaseEmailBackend):
    """
    An email backend for use during test sessions.

    The test connection stores email messages in a dummy outbox,
    rather than sending them out on the wire.

    The dummy outbox is accessible through the outbox instance attribute.
    """

    # RemovedInDjango70Warning: *args. (The only supported posarg will be
    # removed from BaseEmailBackend in Django 7.0.)
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not hasattr(mail, "outbox"):
            mail.outbox = []

    def send_messages(self, messages):
        """Redirect messages to the dummy outbox"""
        msg_count = 0
        for message in messages:
            message.message()  # Trigger header validation.
            msg_copy = copy.deepcopy(message)
            msg_copy.sent_using = self.alias
            mail.outbox.append(msg_copy)
            msg_count += 1
        return msg_count



from django.db import models


class Classroom(models.Model):
    pass


class Lesson(models.Model):
    classroom = models.ForeignKey(Classroom, on_delete=models.CASCADE)


class VeryLongNameModel(models.Model):
    class Meta:
        db_table = "long_db_table_that_should_be_truncated_before_checking"



import re
from contextlib import nullcontext

from django.conf import DEPRECATED_EMAIL_SETTINGS, EMAIL_SETTING_DEPRECATED_MSG
from django.core.mail.deprecation import NO_DEFAULT_MAILER_WARNING
from django.test import ignore_warnings, override_settings
from django.utils.deprecation import RemovedInDjango70Warning


# RemovedInDjango70Warning.
class override_deprecated_email_settings(override_settings):
    """Override settings, ignoring warnings for deprecated email settings.

    Like override_settings(), but suppresses deprecation warnings related to
    defining the overridden settings. Other settings can be included.

    Warnings are ignored only while installing and restoring the settings
    overrides, not for code within the context.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        deprecated_names = [
            name for name in kwargs if name in DEPRECATED_EMAIL_SETTINGS
        ]
        if deprecated_names:
            assert "{name}" in EMAIL_SETTING_DEPRECATED_MSG
            deprecated_names_re = r"|".join(
                re.escape(name) for name in deprecated_names
            )
            message_re = re.escape(EMAIL_SETTING_DEPRECATED_MSG).replace(
                re.escape("{name}"), rf"(?:{deprecated_names_re})"
            )
            self.ignore_warnings = ignore_warnings(
                category=RemovedInDjango70Warning, message=message_re
            )
        else:
            self.ignore_warnings = nullcontext()

    def enable(self):
        with self.ignore_warnings:
            super().enable()

    def disable(self):
        with self.ignore_warnings:
            super().disable()


# RemovedInDjango70Warning: Remove this helper and all uses of it.
def ignore_no_default_mailer_warning():
    return ignore_warnings(
        category=RemovedInDjango70Warning,
        message=re.escape(NO_DEFAULT_MAILER_WARNING),
    )



from django.apps import apps
from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.core.exceptions import FieldDoesNotExist
from django.db.models import CharField, Field, ForeignObjectRel, ManyToManyField
from django.db.models.options import EMPTY_RELATION_TREE, IMMUTABLE_WARNING
from django.test import SimpleTestCase, override_settings

from .models import (
    AbstractPerson,
    BasePerson,
    Child,
    CommonAncestor,
    FirstParent,
    Person,
    ProxyPerson,
    Relating,
    Relation,
    SecondParent,
    Swappable,
)
from .results import TEST_RESULTS


class OptionsBaseTests(SimpleTestCase):
    def _map_related_query_names(self, res):
        return tuple((o.name, m) for o, m in res)

    def _map_names(self, res):
        return tuple((f.name, m) for f, m in res)

    def _model(self, current_model, field):
        model = field.model._meta.concrete_model
        return None if model == current_model else model

    def _details(self, current_model, relation):
        direct = isinstance(relation, (Field, GenericForeignKey))
        model = relation.model._meta.concrete_model
        if model == current_model:
            model = None

        field = relation if direct else relation.field
        return (
            relation,
            model,
            direct,
            bool(field.many_to_many),
        )  # many_to_many can be None


class GetFieldsTests(OptionsBaseTests):
    def test_get_fields_is_immutable(self):
        msg = IMMUTABLE_WARNING % "get_fields()"
        for _ in range(2):
            # Running unit test twice to ensure both non-cached and cached
            # result are immutable.
            fields = Person._meta.get_fields()
            with self.assertRaisesMessage(AttributeError, msg):
                fields += ["errors"]


class LabelTests(OptionsBaseTests):
    def test_label(self):
        for model, expected_result in TEST_RESULTS["labels"].items():
            self.assertEqual(model._meta.label, expected_result)

    def test_label_lower(self):
        for model, expected_result in TEST_RESULTS["lower_labels"].items():
            self.assertEqual(model._meta.label_lower, expected_result)


class DataTests(OptionsBaseTests):
    def test_fields(self):
        for model, expected_result in TEST_RESULTS["fields"].items():
            fields = model._meta.fields
            self.assertEqual([f.attname for f in fields], expected_result)

    def test_local_fields(self):
        def is_data_field(f):
            return isinstance(f, Field) and not f.many_to_many

        for model, expected_result in TEST_RESULTS["local_fields"].items():
            fields = model._meta.local_fields
            self.assertEqual([f.attname for f in fields], expected_result)
            for f in fields:
                self.assertEqual(f.model, model)
                self.assertTrue(is_data_field(f))

    def test_local_concrete_fields(self):
        for model, expected_result in TEST_RESULTS["local_concrete_fields"].items():
            fields = model._meta.local_concrete_fields
            self.assertEqual([f.attname for f in fields], expected_result)
            for f in fields:
                self.assertIsNotNone(f.column)


class M2MTests(OptionsBaseTests):
    def test_many_to_many(self):
        for model, expected_result in TEST_RESULTS["many_to_many"].items():
            fields = model._meta.many_to_many
            self.assertEqual([f.attname for f in fields], expected_result)
            for f in fields:
                self.assertTrue(f.many_to_many and f.is_relation)

    def test_many_to_many_with_model(self):
        for model, expected_result in TEST_RESULTS["many_to_many_with_model"].items():
            models = [self._model(model, field) for field in model._meta.many_to_many]
            self.assertEqual(models, expected_result)


class RelatedObjectsTests(OptionsBaseTests):
    def key_name(self, r):
        return r[0]

    def test_related_objects(self):
        result_key = "get_all_related_objects_with_model"
        for model, expected in TEST_RESULTS[result_key].items():
            objects = [
                (field, self._model(model, field))
                for field in model._meta.get_fields()
                if field.auto_created and not field.concrete
            ]
            self.assertEqual(
                sorted(self._map_related_query_names(objects), key=self.key_name),
                sorted(expected, key=self.key_name),
            )

    def test_related_objects_local(self):
        result_key = "get_all_related_objects_with_model_local"
        for model, expected in TEST_RESULTS[result_key].items():
            objects = [
                (field, self._model(model, field))
                for field in model._meta.get_fields(include_parents=False)
                if field.auto_created and not field.concrete
            ]
            self.assertEqual(
                sorted(self._map_related_query_names(objects), key=self.key_name),
                sorted(expected, key=self.key_name),
            )

    def test_related_objects_include_hidden(self):
        result_key = "get_all_related_objects_with_model_hidden"
        for model, expected in TEST_RESULTS[result_key].items():
            objects = [
                (field, self._model(model, field))
                for field in model._meta.get_fields(include_hidden=True)
                if field.auto_created and not field.concrete
            ]
            self.assertEqual(
                sorted(self._map_names(objects), key=self.key_name),
                sorted(expected, key=self.key_name),
            )

    def test_related_objects_include_hidden_local_only(self):
        result_key = "get_all_related_objects_with_model_hidden_local"
        for model, expected in TEST_RESULTS[result_key].items():
            objects = [
                (field, self._model(model, field))
                for field in model._meta.get_fields(
                    include_hidden=True, include_parents=False
                )
                if field.auto_created and not field.concrete
            ]
            self.assertEqual(
                sorted(self._map_names(objects), key=self.key_name),
                sorted(expected, key=self.key_name),
            )


class PrivateFieldsTests(OptionsBaseTests):
    def test_private_fields(self):
        for model, expected_names in TEST_RESULTS["private_fields"].items():
            objects = model._meta.private_fields
            self.assertEqual(sorted(f.name for f in objects), sorted(expected_names))


class GetFieldByNameTests(OptionsBaseTests):
    def test_get_data_field(self):
        field_info = self._details(Person, Person._meta.get_field("data_abstract"))
        self.assertEqual(field_info[1:], (BasePerson, True, False))
        self.assertIsInstance(field_info[0], CharField)

    def test_get_m2m_field(self):
        field_info = self._details(Person, Person._meta.get_field("m2m_base"))
        self.assertEqual(field_info[1:], (BasePerson, True, True))
        self.assertIsInstance(field_info[0], ManyToManyField)

    def test_get_related_object(self):
        field_info = self._details(
            Person, Person._meta.get_field("relating_baseperson")
        )
        self.assertEqual(field_info[1:], (BasePerson, False, False))
        self.assertIsInstance(field_info[0], ForeignObjectRel)

    def test_get_related_m2m(self):
        field_info = self._details(Person, Person._meta.get_field("relating_people"))
        self.assertEqual(field_info[1:], (None, False, True))
        self.assertIsInstance(field_info[0], ForeignObjectRel)

    def test_get_generic_relation(self):
        field_info = self._details(
            Person, Person._meta.get_field("generic_relation_base")
        )
        self.assertEqual(field_info[1:], (None, True, False))
        self.assertIsInstance(field_info[0], GenericRelation)

    def test_get_fields_only_searches_forward_on_apps_not_ready(self):
        opts = Person._meta
        # If apps registry is not ready, get_field() searches over only
        # forward fields.
        opts.apps.models_ready = False
        try:
            # 'data_abstract' is a forward field, and therefore will be found
            self.assertTrue(opts.get_field("data_abstract"))
            msg = (
                "Person has no field named 'relating_baseperson'. The app "
                "cache isn't ready yet, so if this is an auto-created related "
                "field, it won't be available yet."
            )
            # 'data_abstract' is a reverse field, and will raise an exception
            with self.assertRaisesMessage(FieldDoesNotExist, msg):
                opts.get_field("relating_baseperson")
        finally:
            opts.apps.models_ready = True


class VerboseNameRawTests(SimpleTestCase):
    def test_string(self):
        # Clear cached property.
        Relation._meta.__dict__.pop("verbose_name_raw", None)
        self.assertEqual(Relation._meta.verbose_name_raw, "relation")

    def test_gettext(self):
        Person._meta.__dict__.pop("verbose_name_raw", None)
        self.assertEqual(Person._meta.verbose_name_raw, "Person")


class SwappedTests(SimpleTestCase):
    def test_plain_model_none(self):
        self.assertIsNone(Relation._meta.swapped)

    def test_unset(self):
        self.assertIsNone(Swappable._meta.swapped)

    def test_set_and_unset(self):
        with override_settings(MODEL_META_TESTS_SWAPPED="model_meta.Relation"):
            self.assertEqual(Swappable._meta.swapped, "model_meta.Relation")
        self.assertIsNone(Swappable._meta.swapped)

    def test_setting_none(self):
        with override_settings(MODEL_META_TESTS_SWAPPED=None):
            self.assertIsNone(Swappable._meta.swapped)

    def test_setting_non_label(self):
        with override_settings(MODEL_META_TESTS_SWAPPED="not-a-label"):
            self.assertEqual(Swappable._meta.swapped, "not-a-label")

    def test_setting_self(self):
        with override_settings(MODEL_META_TESTS_SWAPPED="model_meta.swappable"):
            self.assertIsNone(Swappable._meta.swapped)


class RelationTreeTests(SimpleTestCase):
    all_models = (Relation, AbstractPerson, BasePerson, Person, ProxyPerson, Relating)

    def setUp(self):
        apps.clear_cache()

    def test_clear_cache_clears_relation_tree(self):
        # The apps.clear_cache is setUp() should have deleted all trees.
        # Exclude abstract models that are not included in the Apps registry
        # and have no cache.
        all_models_with_cache = (m for m in self.all_models if not m._meta.abstract)
        for m in all_models_with_cache:
            self.assertNotIn("_relation_tree", m._meta.__dict__)

    def test_first_relation_tree_access_populates_all(self):
        # On first access, relation tree should have populated cache.
        self.assertTrue(self.all_models[0]._meta._relation_tree)

        # AbstractPerson does not have any relations, so relation_tree
        # should just return an EMPTY_RELATION_TREE.
        self.assertEqual(AbstractPerson._meta._relation_tree, EMPTY_RELATION_TREE)

        # All the other models should already have their relation tree
        # in the internal __dict__ .
        all_models_but_abstractperson = (
            m for m in self.all_models if m is not AbstractPerson
        )
        for m in all_models_but_abstractperson:
            self.assertIn("_relation_tree", m._meta.__dict__)

    def test_relations_related_objects(self):
        # Testing non hidden related objects
        self.assertEqual(
            sorted(
                field.related_query_name()
                for field in Relation._meta._relation_tree
                if not field.remote_field.field.remote_field.hidden
            ),
            sorted(
                [
                    "fk_abstract_rel",
                    "fk_base_rel",
                    "fk_concrete_rel",
                    "fo_abstract_rel",
                    "fo_base_rel",
                    "fo_concrete_rel",
                    "m2m_abstract_rel",
                    "m2m_base_rel",
                    "m2m_concrete_rel",
                ]
            ),
        )
        # Testing hidden related objects
        self.assertEqual(
            sorted(
                field.related_query_name() for field in BasePerson._meta._relation_tree
            ),
            sorted(
                [
                    "+",
                    "_model_meta_relating_basepeople_hidden_+",
                    "BasePerson_following_abstract+",
                    "BasePerson_following_abstract+",
                    "BasePerson_following_base+",
                    "BasePerson_following_base+",
                    "BasePerson_friends_abstract+",
                    "BasePerson_friends_abstract+",
                    "BasePerson_friends_base+",
                    "BasePerson_friends_base+",
                    "BasePerson_m2m_abstract+",
                    "BasePerson_m2m_base+",
                    "Relating_basepeople+",
                    "Relating_basepeople_hidden+",
                    "followers_abstract",
                    "followers_base",
                    "friends_abstract_rel_+",
                    "friends_base_rel_+",
                    "person",
                    "relating_basepeople",
                    "relating_baseperson",
                ]
            ),
        )
        self.assertEqual(
            [
                field.related_query_name()
                for field in AbstractPerson._meta._relation_tree
            ],
            [],
        )


class AllParentsTests(SimpleTestCase):
    def test_all_parents(self):
        self.assertEqual(CommonAncestor._meta.all_parents, ())
        self.assertEqual(FirstParent._meta.all_parents, (CommonAncestor,))
        self.assertEqual(SecondParent._meta.all_parents, (CommonAncestor,))
        self.assertEqual(
            Child._meta.all_parents,
            (FirstParent, SecondParent, CommonAncestor),
        )

    def test_get_parent_list(self):
        self.assertEqual(Child._meta.get_parent_list(), list(Child._meta.all_parents))


class PropertyNamesTests(SimpleTestCase):
    def test_person(self):
        # Instance only descriptors don't appear in _property_names.
        self.assertEqual(BasePerson().test_instance_only_descriptor, 1)
        with self.assertRaisesMessage(AttributeError, "Instance only"):
            AbstractPerson.test_instance_only_descriptor
        self.assertEqual(
            AbstractPerson._meta._property_names, frozenset(["pk", "test_property"])
        )


class ReturningFieldsTests(SimpleTestCase):
    def test_pk(self):
        self.assertEqual(Relation._meta.db_returning_fields, [Relation._meta.pk])


class AbstractModelTests(SimpleTestCase):
    def test_abstract_model_not_instantiated(self):
        msg = "Abstract models cannot be instantiated."
        with self.assertRaisesMessage(TypeError, msg):
            AbstractPerson()



from .tenant import Comment, Post, PostDbDefault, Tenant, TimeStamped, Token, User

__all__ = [
    "Comment",
    "Post",
    "PostDbDefault",
    "Tenant",
    "TimeStamped",
    "Token",
    "User",
]



import functools
from importlib import import_module

from django.core.exceptions import ViewDoesNotExist
from django.utils.module_loading import module_has_submodule


@functools.cache
def get_callable(lookup_view):
    """
    Return a callable corresponding to lookup_view.
    * If lookup_view is already a callable, return it.
    * If lookup_view is a string import path that can be resolved to a
      callable, import that callable and return it, otherwise raise an
      exception (ImportError or ViewDoesNotExist).
    """
    if callable(lookup_view):
        return lookup_view

    if not isinstance(lookup_view, str):
        raise ViewDoesNotExist(
            "'%s' is not a callable or a dot-notation path" % lookup_view
        )

    mod_name, func_name = get_mod_func(lookup_view)
    if not func_name:  # No '.' in lookup_view
        raise ImportError(
            "Could not import '%s'. The path must be fully qualified." % lookup_view
        )

    try:
        mod = import_module(mod_name)
    except ImportError:
        parentmod, submod = get_mod_func(mod_name)
        if submod and not module_has_submodule(import_module(parentmod), submod):
            raise ViewDoesNotExist(
                "Could not import '%s'. Parent module %s does not exist."
                % (lookup_view, mod_name)
            )
        else:
            raise
    else:
        try:
            view_func = getattr(mod, func_name)
        except AttributeError:
            raise ViewDoesNotExist(
                "Could not import '%s'. View does not exist in module %s."
                % (lookup_view, mod_name)
            )
        else:
            if not callable(view_func):
                raise ViewDoesNotExist(
                    "Could not import '%s.%s'. View is not callable."
                    % (mod_name, func_name)
                )
            return view_func


def get_mod_func(callback):
    # Convert 'django.views.news.stories.story_detail' to
    # ['django.views.news.stories', 'story_detail']
    try:
        dot = callback.rindex(".")
    except ValueError:
        return callback, ""
    return callback[:dot], callback[dot + 1 :]



from django.core.checks import Error, Warning, registry
from django.core.checks.mail import (
    check_mailers_default_alias,
    check_mailers_production_backend,
)
from django.test import SimpleTestCase, override_settings


class MailersDefaultAliasCheckTests(SimpleTestCase):
    def test_mailers_not_defined(self):
        self.assertEqual(check_mailers_default_alias(None), [])

    @override_settings(
        MAILERS={
            "default": {
                "BACKEND": "django.core.mail.backends.smtp.EmailBackend",
            }
        }
    )
    def test_mailers_default_alias_configured(self):
        self.assertEqual(check_mailers_default_alias(None), [])

    @override_settings(MAILERS={})
    def test_mailers_empty(self):
        self.assertEqual(
            check_mailers_default_alias(None),
            [
                Warning(
                    "Your MAILERS setting has no 'default' entry. Sending email "
                    "without a valid mailer will fail.",
                    hint="Add a 'default' entry to MAILERS.",
                    id="mail.W001",
                )
            ],
        )

    @override_settings(
        MAILERS={
            "secondary": {
                "BACKEND": "django.core.mail.backends.smtp.EmailBackend",
            }
        }
    )
    def test_mailers_without_default_alias(self):
        self.assertEqual(
            check_mailers_default_alias(None),
            [
                Warning(
                    "Your MAILERS setting has no 'default' entry. Sending email "
                    "without a valid mailer will fail.",
                    hint=(
                        "Add a 'default' entry to MAILERS, or pass 'using' when "
                        "sending email."
                    ),
                    id="mail.W001",
                )
            ],
        )


class MailersProductionBackendTests(SimpleTestCase):
    def test_is_deployment_only(self):
        self.assertIn(
            check_mailers_production_backend, registry.registry.deployment_checks
        )
        self.assertNotIn(
            check_mailers_production_backend, registry.registry.registered_checks
        )

    def test_mailers_not_defined(self):
        self.assertEqual(check_mailers_production_backend(None), [])

    def test_production_backends(self):
        production_backends = [
            "django.core.mail.backends.smtp.EmailBackend",
            "any.third.party.EmailBackend",
        ]
        for backend in production_backends:
            with (
                self.subTest(backend=backend),
                self.settings(MAILERS={"default": {"BACKEND": backend}}),
            ):
                self.assertEqual(check_mailers_production_backend(None), [])

    def test_non_production_backends(self):
        hint = (
            "Use a production-ready email backend, such as the SMTP backend, "
            "otherwise email will not be sent."
        )
        for backend_type in ["console", "dummy", "filebased", "locmem"]:
            backend = f"django.core.mail.backends.{backend_type}.EmailBackend"
            msg = (
                f"Your MAILERS setting uses a development-only email backend in "
                f"the 'default' entry ({backend})."
            )
            with (
                self.subTest(backend=backend),
                self.settings(MAILERS={"default": {"BACKEND": backend}}),
            ):
                self.assertEqual(
                    check_mailers_production_backend(None),
                    [Error(msg, hint=hint, id="mail.E001")],
                )

    @override_settings(
        MAILERS={
            "default": {"BACKEND": "django.core.mail.backends.smtp.EmailBackend"},
            "secondary": {"BACKEND": "django.core.mail.backends.console.EmailBackend"},
        }
    )
    def test_only_applies_to_default_mailer(self):
        self.assertEqual(
            check_mailers_production_backend(None),
            [],
        )



from django.core.mail import InvalidMailer, MailerDoesNotExist, mailers
from django.core.mail.backends import locmem, smtp
from django.test import SimpleTestCase, override_settings

from . import (
    ignore_no_default_mailer_warning,
    override_deprecated_email_settings,
)
from .custombackend import OptionsCapturingBackend


class MailersTests(SimpleTestCase):
    def setUp(self):
        self.addCleanup(OptionsCapturingBackend.reset)

    @override_settings(
        MAILERS={
            "default": {"BACKEND": "django.core.mail.backends.locmem.EmailBackend"},
            "custom": {"BACKEND": "django.core.mail.backends.locmem.EmailBackend"},
        }
    )
    def test_getitem(self):
        with self.subTest("defined mailers"):
            self.assertEqual(mailers["default"].alias, "default")
            self.assertEqual(mailers["custom"].alias, "custom")

        with self.subTest("missing mailer"):
            msg = "The mailer 'unknown' is not configured."
            with self.assertRaisesMessage(MailerDoesNotExist, msg):
                _ = mailers["unknown"]

        with self.subTest("raises KeyError"):
            # mail.mailers is a mapping, so unknown keys raise KeyError.
            # (MailerDoesNotExist must be a KeyError.)
            with self.assertRaises(KeyError):
                _ = mailers["unknown"]

    @override_settings(
        MAILERS={
            "one": {"BACKEND": "mail.custombackend.OptionsCapturingBackend"},
            "two": {"BACKEND": "mail.custombackend.OptionsCapturingBackend"},
            "three": {"BACKEND": "mail.custombackend.OptionsCapturingBackend"},
        }
    )
    def test_contains(self):
        self.assertIn("two", mailers)
        self.assertNotIn("zero", mailers)
        self.assertNotIn(None, mailers)
        # __contains__() does not construct any backend instance.
        self.assertEqual(OptionsCapturingBackend.init_kwargs, [])

    @override_settings(
        MAILERS={
            "one": {"BACKEND": "mail.custombackend.OptionsCapturingBackend"},
            "two": {"BACKEND": "mail.custombackend.OptionsCapturingBackend"},
            "three": {"BACKEND": "mail.custombackend.OptionsCapturingBackend"},
        }
    )
    def test_iter(self):
        self.assertEqual(list(mailers), ["one", "two", "three"])
        # __iter__() does not construct any backend instance.
        self.assertEqual(OptionsCapturingBackend.init_kwargs, [])

    @override_settings(
        MAILERS={
            "default": {"BACKEND": "django.core.mail.backends.locmem.EmailBackend"},
            "custom": {"BACKEND": "django.core.mail.backends.locmem.EmailBackend"},
        }
    )
    def test_get(self):
        with self.subTest("defined mailers"):
            self.assertEqual(mailers.get("default").alias, "default")
            self.assertEqual(mailers.get("custom").alias, "custom")

        with self.subTest("missing mailer"):
            with self.subTest("no default arg"):
                self.assertIsNone(mailers.get("unknown"))
            with self.subTest("positional default arg"):
                self.assertEqual(mailers.get("unknown", "foo"), "foo")
            with self.subTest("keyword default arg"):
                self.assertEqual(mailers.get("unknown", default="foo"), "foo")

    @override_settings(
        MAILERS={
            "default": {"BACKEND": "django.core.mail.backends.locmem.EmailBackend"}
        }
    )
    def test_default_mailer_property(self):
        backend = mailers.default
        self.assertEqual(backend.alias, "default")

    # RemovedInDjango70Warning: remove override_settings (but keep the test).
    # (MAILERS={} becomes the default in Django 7.0.)
    @override_settings(MAILERS={})
    def test_default_mailers(self):
        msg = "The mailer 'default' is not configured."
        with (
            self.subTest('mailers["default"]'),
            self.assertRaisesMessage(MailerDoesNotExist, msg),
        ):
            _ = mailers["default"]
        with (
            self.subTest("mailers.default"),
            self.assertRaisesMessage(MailerDoesNotExist, msg),
        ):
            _ = mailers.default

        with self.subTest("mailers.get()"):
            self.assertIsNone(mailers.get("default"))
        with self.subTest("mailers.__contains__()"):
            self.assertIs("default" in mailers, False)
        with self.subTest("mailers.__iter__()"):
            self.assertEqual(list(mailers), [])

    @override_settings(
        MAILERS={"custom": {"BACKEND": "django.core.mail.backends.locmem.EmailBackend"}}
    )
    def test_default_mailer_not_required(self):
        self.assertEqual(mailers["custom"].alias, "custom")
        msg = "The mailer 'default' is not configured."
        with self.assertRaisesMessage(MailerDoesNotExist, msg):
            _ = mailers.default

    @override_settings(
        MAILERS={"custom": {"BACKEND": "django.core.mail.backends.locmem.EmailBackend"}}
    )
    def test_instances_not_cached(self):
        self.assertIsNot(mailers["custom"], mailers["custom"])

    @override_settings(MAILERS={"custom": {"OPTIONS": {"host": "localhost"}}})
    def test_default_backend_is_smtp(self):
        # Omitting "BACKEND" gives the SMTP EmailBackend.
        backend = mailers["custom"]
        self.assertIsInstance(backend, smtp.EmailBackend)

    @override_settings(
        MAILERS={"default": {"BACKEND": "mail.custombackend.EmailBackend"}}
    )
    def test_custom_backend(self):
        backend = mailers.default
        self.assertTrue(hasattr(backend, "test_outbox"))

    @override_settings(MAILERS={"custom": {"BACKEND": "foo.bar"}})
    def test_invalid_backend(self):
        msg = (
            "MAILERS['custom']: Could not find BACKEND 'foo.bar': No "
            "module named 'foo'"
        )
        with self.assertRaisesMessage(InvalidMailer, msg):
            _ = mailers["custom"]

    @override_settings(
        MAILERS={
            "custom": {
                "BACKEND": "mail.custombackend.OptionsCapturingBackend",
                "OPTIONS": {"one": 1, "false": False, "foo": "bar"},
            }
        }
    )
    def test_options_are_provided_to_backend_init(self):
        _ = mailers["custom"]
        self.assertEqual(
            OptionsCapturingBackend.init_kwargs[0],
            {"alias": "custom", "one": 1, "false": False, "foo": "bar"},
        )

    @override_settings(
        MAILERS={
            "custom": {
                "BACKEND": "django.core.mail.backends.smtp.EmailBackend",
                "OPTIONS": {"alias": "imposter", "host": "localhost"},
            }
        }
    )
    def test_alias_is_invalid_option(self):
        msg = "MAILERS['custom']: OPTIONS must not define 'alias'."
        with self.assertRaisesMessage(InvalidMailer, msg):
            _ = mailers["custom"]
        with self.assertRaises(MailerDoesNotExist):
            _ = mailers["imposter"]

    @override_settings(
        MAILERS={
            "custom": {
                "BACKEND": "django.core.mail.backends.locmem.EmailBackend",
                "OPTIONS": {"unknown": "foo"},
            }
        }
    )
    def test_unknown_options(self):
        # This error message actually comes from BaseEmailBackend.
        msg = "MAILERS['custom']: Unknown options 'unknown'."
        with self.assertRaisesMessage(InvalidMailer, msg):
            _ = mailers["custom"]

    def test_does_not_exist_is_limited_purpose(self):
        # Code that wants to send email only when it has been configured will
        # trap and ignore MailerDoesNotExist. If that error is used to
        # report anything other than a missing alias key in MAILERS,
        # unrelated configuration errors may be incorrectly silenced. The
        # error's constructor is designed to discourage other uses.
        msg = "MailerDoesNotExist.__init__() takes 1 positional argument"
        with self.assertRaisesMessage(TypeError, msg):
            MailerDoesNotExist("Some other configuration problem")


# RemovedInDjango70Warning.
class MailersCompatibilityTests(SimpleTestCase):
    """mailers.default is usable even when MAILERS is not defined."""

    @override_deprecated_email_settings(
        EMAIL_BACKEND="mail.custombackend.OptionsCapturingBackend"
    )
    def test_default_mailer_with_deprecated_settings(self):
        self.addCleanup(OptionsCapturingBackend.reset)
        backend = mailers.default
        self.assertIsNone(backend.alias)
        self.assertIsInstance(backend, OptionsCapturingBackend)
        self.assertNotIn("alias", OptionsCapturingBackend.init_kwargs[0])

    @ignore_no_default_mailer_warning()
    def test_default_mailer_with_no_settings(self):
        backend = mailers.default
        # Django's test runner changes the default EMAIL_BACKEND to locmem.
        self.assertIsInstance(backend, locmem.EmailBackend)
        # In compatibility mode, backends are constructed with no 'alias' arg.
        self.assertIsNone(backend.alias)

    def test_unknown_mailer_with_no_settings(self):
        # Compatibility only applies to the default mailer.
        msg = "The mailer 'unknown' is not configured."
        with self.assertRaisesMessage(MailerDoesNotExist, msg):
            _ = mailers["unknown"]



from django.contrib.auth.hashers import make_password
from django.db import models


class CustomUserNoNaturalKeyManager(models.Manager):
    def create_superuser(self, email, password=None, **extra_fields):
        user = self.model(email=email, is_superuser=True, **extra_fields)
        user.password = make_password(password)
        user.save(using=self._db)
        return user


class CustomUserNoNaturalKey(models.Model):
    email = models.EmailField(max_length=255, unique=True)
    password = models.CharField(max_length=128)
    is_superuser = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []

    objects = CustomUserNoNaturalKeyManager()



from django import template
from django.contrib.admin.options import EMPTY_VALUE_STRING
from django.contrib.admin.utils import display_for_value
from django.template.defaultfilters import _walk_items, stringfilter
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ngettext

register = template.Library()


@register.filter
@stringfilter
def to_object_display_value(value):
    return display_for_value(str(value), EMPTY_VALUE_STRING)


@register.filter(is_safe=True, needs_autoescape=True)
def truncated_unordered_list(value, max_items, autoescape=True):
    """
    Render an unordered list, showing at most ``max_items`` items and a
    "...and N more objects." item at the end.

    Usage::

        {{ deleted_objects|truncated_unordered_list:100 }}
    """

    has_unlimited_items = max_items is None
    if not has_unlimited_items:
        max_items = int(max_items)
        if max_items <= 0:
            return mark_safe("")

    if autoescape:
        escaper = conditional_escape
    else:

        def escaper(x):
            return x

    item_count = 0

    def list_formatter(item_list, tabs=1):
        nonlocal item_count
        indent = "\t" * tabs
        output = []
        for item, children in _walk_items(item_list):
            sublist = ""
            item_count += 1
            should_display_item = has_unlimited_items or 0 < item_count <= max_items
            if children:
                sublist = "\n%s<ul>\n%s\n%s</ul>\n%s" % (
                    indent,
                    list_formatter(children, tabs + 1),
                    indent,
                    indent,
                )

            if should_display_item:
                output.append("%s<li>%s%s</li>" % (indent, escaper(item), sublist))

        return "\n".join(output)

    rendered_object_list = list_formatter(value)
    remaining_objects_message = ""

    if not has_unlimited_items and item_count > max_items:
        remaining_object_count = item_count - max_items
        remaining_objects_message = "\n\t<li>%s</li>" % (
            ngettext(
                "…and %(count)d more object.",
                "…and %(count)d more objects.",
                remaining_object_count,
            )
            % {"count": remaining_object_count}
        )

    return mark_safe(rendered_object_list + remaining_objects_message)



from django.contrib.gis.geos import prototypes as capi
from django.contrib.gis.geos.coordseq import GEOSCoordSeq
from django.contrib.gis.geos.error import GEOSException
from django.contrib.gis.geos.geometry import GEOSGeometry, LinearGeometryMixin
from django.contrib.gis.geos.point import Point
from django.contrib.gis.shortcuts import numpy


class LineString(LinearGeometryMixin, GEOSGeometry):
    _init_func = capi.create_linestring
    _minlength = 2
    has_cs = True

    def __init__(self, *args, **kwargs):
        """
        Initialize on the given sequence: may take lists, tuples, NumPy arrays
        of X,Y pairs, or Point objects. If Point objects are used, ownership is
        _not_ transferred to the LineString object.

        Examples:
         ls = LineString((1, 1), (2, 2))
         ls = LineString([(1, 1), (2, 2)])
         ls = LineString(array([(1, 1), (2, 2)]))
         ls = LineString(Point(1, 1), Point(2, 2))
        """
        # If only one argument provided, set the coords array appropriately
        if len(args) == 1:
            coords = args[0]
        else:
            coords = args

        if not (
            isinstance(coords, (tuple, list))
            or numpy
            and isinstance(coords, numpy.ndarray)
        ):
            raise TypeError("Invalid initialization input for LineStrings.")

        # If SRID was passed in with the keyword arguments
        srid = kwargs.get("srid")

        ncoords = len(coords)
        if not ncoords:
            super().__init__(self._init_func(None), srid=srid)
            return

        if ncoords < self._minlength:
            raise ValueError(
                "%s requires at least %d points, got %s."
                % (
                    self.__class__.__name__,
                    self._minlength,
                    ncoords,
                )
            )

        numpy_coords = not isinstance(coords, (tuple, list))
        if numpy_coords:
            shape = coords.shape  # Using numpy's shape.
            if len(shape) != 2:
                raise TypeError("Too many dimensions.")
            self._checkdim(shape[1])
            ndim = shape[1]
        else:
            # Getting the number of coords and the number of dimensions, which
            #  must stay the same, e.g., no LineString((1, 2), (1, 2, 3)).
            ndim = None
            # Incrementing through each of the coordinates and verifying
            for coord in coords:
                if not isinstance(coord, (tuple, list, Point)):
                    raise TypeError(
                        "Each coordinate should be a sequence (list or tuple)"
                    )

                if ndim is None:
                    ndim = len(coord)
                    self._checkdim(ndim)
                elif len(coord) != ndim:
                    raise TypeError("Dimension mismatch.")

        # Creating a coordinate sequence object because it is easier to
        # set the points using its methods.
        cs = GEOSCoordSeq(capi.create_cs(ncoords, ndim), z=bool(ndim == 3))
        point_setter = cs._set_point_3d if ndim == 3 else cs._set_point_2d

        for i in range(ncoords):
            if numpy_coords:
                point_coords = coords[i, :]
            elif isinstance(coords[i], Point):
                point_coords = coords[i].tuple
            else:
                point_coords = coords[i]
            point_setter(i, point_coords)

        # Calling the base geometry initialization with the returned pointer
        #  from the function.
        super().__init__(self._init_func(cs.ptr), srid=srid)

    def __iter__(self):
        "Allow iteration over this LineString."
        for i in range(len(self)):
            yield self[i]

    def __len__(self):
        "Return the number of points in this LineString."
        return len(self._cs)

    def _get_single_external(self, index):
        return self._cs[index]

    _get_single_internal = _get_single_external

    def _set_list(self, length, items):
        ndim = self._cs.dims
        hasz = self._cs.hasz  # I don't understand why these are different
        srid = self.srid

        # create a new coordinate sequence and populate accordingly
        cs = GEOSCoordSeq(capi.create_cs(length, ndim), z=hasz)
        for i, c in enumerate(items):
            cs[i] = c

        ptr = self._init_func(cs.ptr)
        if ptr:
            capi.destroy_geom(self.ptr)
            self.ptr = ptr
            if srid is not None:
                self.srid = srid
            self._post_init()
        else:
            # can this happen?
            raise GEOSException("Geometry resulting from slice deletion was invalid.")

    def _set_single(self, index, value):
        self._cs[index] = value

    def _checkdim(self, dim):
        if dim not in (2, 3):
            raise TypeError("Dimension mismatch.")

    # #### Sequence Properties ####
    @property
    def tuple(self):
        "Return a tuple version of the geometry from the coordinate sequence."
        return self._cs.tuple

    coords = tuple

    def _listarr(self, func):
        """
        Return a sequence (list) corresponding with the given function.
        Return a numpy array if possible.
        """
        lst = [func(i) for i in range(len(self))]
        if numpy:
            return numpy.array(lst)  # ARRRR!
        else:
            return lst

    @property
    def array(self):
        "Return a numpy array for the LineString."
        return self._listarr(self._cs.__getitem__)

    @property
    def x(self):
        "Return a list or numpy array of the X variable."
        return self._listarr(self._cs.getX)

    @property
    def y(self):
        "Return a list or numpy array of the Y variable."
        return self._listarr(self._cs.getY)

    @property
    def z(self):
        "Return a list or numpy array of the Z variable."
        if not self.hasz:
            return None
        else:
            return self._listarr(self._cs.getZ)


# LinearRings are LineStrings used within Polygons.
class LinearRing(LineString):
    _minlength = 4
    _init_func = capi.create_linearring

    @property
    def is_counterclockwise(self):
        if self.empty:
            raise ValueError("Orientation of an empty LinearRing cannot be determined.")
        return self._cs.is_counterclockwise
